USE AdventureWorks2014;
GO

SET ANSI_NULLS ON;
SET QUOTED_IDENTIFIER ON;
SET NOCOUNT ON;

IF EXISTS (SELECT * FROM sys.tables WHERE tables.name = 'Dim_Date')
BEGIN
	DROP TABLE dbo.Dim_Date;
END
GO

CREATE TABLE dbo.Dim_Date
(	Calendar_Date DATE NOT NULL CONSTRAINT PK_Dim_Date PRIMARY KEY CLUSTERED, -- The date addressed in this row.
	Calendar_Date_String VARCHAR(10) NOT NULL, -- The VARCHAR formatted date, such as 07/03/2017
	Calendar_Month TINYINT NOT NULL, -- Number from 1-12
	Calendar_Day TINYINT NOT NULL, -- Number from 1 through 31
	Calendar_Year SMALLINT NOT NULL, -- Current year, eg: 2017, 2025, 1984.
	Calendar_Quarter TINYINT NOT NULL, -- 1-4, indicates quarter within the current year.
	Day_Name VARCHAR(9) NOT NULL, -- Name of the day of the week, Sunday...Saturday
	Day_of_Week TINYINT NOT NULL, -- Number from 1-7 (1 = Sunday)
	Day_of_Week_in_Month TINYINT NOT NULL, -- Number from 1-5, indicates for example that it's the Nth saturday of the month.
	Day_of_Week_in_Year TINYINT NOT NULL, -- Number from 1-53, indicates for example that it's the Nth saturday of the year.
	Day_of_Week_in_Quarter TINYINT NOT NULL, -- Number from 1-13, indicates for example that it's the Nth saturday of the quarter.
	Day_of_Quarter TINYINT NOT NULL, -- Number from 1-92, indicates the day # in the quarter.
	Day_of_Year SMALLINT NOT NULL, -- Number from 1-366
	Week_of_Month TINYINT NOT NULL, -- Number from 1-6, indicates the number of week within the current month.
	Week_of_Quarter TINYINT NOT NULL, -- Number from 1-14, indicates the number of week within the current quarter.
	Week_of_Year TINYINT NOT NULL, -- Number from 1-53, indicates the number of week within the current year.
	Month_Name VARCHAR(9) NOT NULL, -- January-December
	First_Date_of_Week DATE NOT NULL, -- Date of the first day of this week.
	Last_Date_of_Week DATE NOT NULL, -- Date of the last day of this week.
	First_Date_of_Month DATE NOT NULL, -- Date of the first day of this month.
	Last_Date_of_Month DATE NOT NULL, -- Date of the last day of this month.
	First_Date_of_Quarter DATE NOT NULL, -- Date of the first day of this quarter.
	Last_Date_of_Quarter DATE NOT NULL, -- Date of the last day of this quarter.
	First_Date_of_Year DATE NOT NULL, -- Date of the first day of this year.
	Last_Date_of_Year DATE NOT NULL, -- Date of the last day of this year.
	Is_Leap_Year BIT NOT NULL, -- 1 if current year is a leap year.
	Days_in_Month TINYINT NOT NULL, -- Number of days in the current month.
	Merchant_Day_of_Year SMALLINT NOT NULL, -- Day of year in the merchant calendar
	Merchant_Week_of_Year SMALLINT NOT NULL, -- Week of year in the merchant calendar
	Merchant_Calendar_Month TINYINT NOT NULL, -- Month of year in the merchant calendar
	Merchant_Calendar_Quarter TINYINT NOT NULL, -- Quarter of year in the merchant calendar
	Merchant_First_Date_of_Week DATE NOT NULL, -- First date of the week in the Merchant calendar
	Merchant_Last_Date_of_Week DATE NOT NULL, -- Last date of the week in the Merchant calendar
	Merchant_First_Date_of_Month DATE NOT NULL, -- First date of the month in the merchant calendar
	Merchant_Last_Date_of_Month DATE NOT NULL, -- Last date of the month in the merchant calendar
	Merchant_First_Date_of_Quarter DATE NOT NULL, -- First date of the quarter in the merchant calendar
	Merchant_Last_Date_of_Quarter DATE NOT NULL, -- Last date of the quarter in the merchant calendar
	Merchant_First_Date_of_Year DATE NOT NULL, -- First date of the year in the merchant calendar
	Merchant_Last_Date_of_Year DATE NOT NULL, -- Last date of the year in the merchant calendar
	ISO_Day_of_Week TINYINT NOT NULL, -- The day of the week in the ISO calendar
	ISO_Day_of_Year SMALLINT NOT NULL, -- Day of year in the ISO calendar
	ISO_Week_of_Year TINYINT NOT NULL, -- Week of year in the ISO calendar
	ISO_First_Date_of_Week DATE NOT NULL, -- First date of the week in the ISO calendar
	ISO_Last_Date_of_Week DATE NOT NULL, -- Last date of the week in the ISO calendar
	ISO_String VARCHAR(10) NOT NULL, -- Standard string to represent an ISO date, using YYYY-wWW-D
	Fiscal_Day_of_Year SMALLINT NOT NULL, -- Day of year in a fiscal calendar that begins on October 1st.
	Fiscal_Week_of_Year SMALLINT NOT NULL, -- Week of year in a fiscal calendar that begins on October 1st.
	Fiscal_Quarter TINYINT NOT NULL, -- 1-4, indicates quarter within the current fiscal year, beginning on October 1st.
	Fiscal_Month TINYINT NOT NULL, -- 1-12, indicates month within the current fiscal year, beginning on October 1st.
	Fiscal_First_Date_of_Year DATE NOT NULL, -- First date of the year in the current fiscal year, beginning on October 1st.
	Fiscal_Last_Date_of_Year DATE NOT NULL -- Last date of the year in the current fiscal year, beginning on October 1st.
);
GO

IF EXISTS (SELECT * FROM sys.procedures WHERE procedures.name = 'Populate_Dim_Date')
BEGIN
	DROP PROCEDURE dbo.Populate_Dim_Date;
END
GO

CREATE PROCEDURE dbo.Populate_Dim_Date
	@Start_Date DATE, -- Start of date range to process
	@End_Date DATE -- End of date range to process
AS
BEGIN
	SET NOCOUNT ON;

	IF @Start_Date IS NULL OR @End_Date IS NULL
	BEGIN
		SELECT 'Start and end dates MUST be provided in order for this stored procedure to work.';
		RETURN;
	END

	IF @Start_Date > @End_Date
	BEGIN
		SELECT 'Start date must be less than or equal to the end date.';
		RETURN;
	END

	-- Remove all old data for the date range provided.
	DELETE FROM dbo.Dim_Date
	WHERE Dim_Date.Calendar_Date BETWEEN @Start_Date AND @End_Date;
	-- These variables dirrectly correspond to columns in Dim_Date
	DECLARE @Date_Counter DATE = @Start_Date;
	DECLARE @Calendar_Date_String VARCHAR(10);
	DECLARE @Calendar_Month TINYINT;
	DECLARE @Calendar_Day TINYINT;
	DECLARE @Calendar_Year SMALLINT;
	DECLARE @Calendar_Quarter TINYINT;
	DECLARE @Day_Name VARCHAR(9);
	DECLARE @Day_of_Week TINYINT;
	DECLARE @Day_of_Week_in_Month TINYINT;
	DECLARE @Day_of_Week_in_Year TINYINT;
	DECLARE @Day_of_Week_in_Quarter TINYINT;
	DECLARE @Day_of_Quarter TINYINT;
	DECLARE @Day_of_Year SMALLINT;
	DECLARE @Week_of_Month TINYINT;
	DECLARE @Week_of_Quarter TINYINT;
	DECLARE @Week_of_Year TINYINT;
	DECLARE @Month_Name VARCHAR(9);
	DECLARE @First_Date_of_Week DATE;
	DECLARE @Last_Date_of_Week DATE;
	DECLARE @First_Date_of_Month DATE;
	DECLARE @Last_Date_of_Month DATE;
	DECLARE @First_Date_of_Quarter DATE;
	DECLARE @Last_Date_of_Quarter DATE;
	DECLARE @First_Date_of_Year DATE;
	DECLARE @Last_Date_of_Year DATE;
	DECLARE @Is_Leap_Year BIT;
	DECLARE @Days_in_Month TINYINT;
	DECLARE @Merchant_Day_of_Year SMALLINT;
	DECLARE @Merchant_Week_of_Year TINYINT;
	DECLARE @Merchant_Calendar_Month TINYINT;
	DECLARE @Merchant_Calendar_Quarter TINYINT;
	DECLARE @Merchant_First_Date_of_Week DATE;
	DECLARE @Merchant_Last_Date_of_Week DATE;
	DECLARE @Merchant_First_Date_of_Month DATE;
	DECLARE @Merchant_Last_Date_of_Month DATE;
	DECLARE @Merchant_First_Date_of_Quarter DATE;
	DECLARE @Merchant_Last_Date_of_Quarter DATE;
	DECLARE @Merchant_First_Date_of_Year DATE;
	DECLARE @Merchant_Last_Date_of_Year DATE;
	DECLARE @ISO_Day_of_Week TINYINT;
	DECLARE @ISO_Day_of_Year SMALLINT;
	DECLARE @ISO_Week_of_Year TINYINT;
	DECLARE @ISO_First_Date_of_Week DATE;
	DECLARE @ISO_Last_Date_of_Week DATE;
	DECLARE @ISO_String VARCHAR(10);
	DECLARE @Fiscal_Day_of_Year SMALLINT;
	DECLARE @Fiscal_Week_of_Year SMALLINT;
	DECLARE @Fiscal_Quarter TINYINT;
	DECLARE @Fiscal_Month TINYINT;
	DECLARE @Fiscal_First_Date_of_Year DATE;
	DECLARE @Fiscal_Last_Date_of_Year DATE;

	-- These variables are used internally within this proc for various calculations
	DECLARE @First_Date_of_Next_Year DATE;
	DECLARE @First_Date_of_Last_Year DATE;
	DECLARE @First_Date_of_Two_Years_Ago DATE;
	DECLARE @Merchant_First_Date_of_Next_Year DATE;
	DECLARE @Merchant_First_Date_of_Last_Year DATE;
	DECLARE @Is_Merchant_Leap_Year BIT;
	DECLARE @ISO_Year SMALLINT;

	WHILE @Date_Counter <= @End_Date
	BEGIN
		SELECT @Calendar_Month = DATEPART(MONTH, @Date_Counter);
		SELECT @Calendar_Day = DATEPART(DAY, @Date_Counter);
		SELECT @Calendar_Year = DATEPART(YEAR, @Date_Counter);
		SELECT @Calendar_Quarter = DATEPART(QUARTER, @Date_Counter);
		SELECT @Calendar_Date_String = CAST(@Calendar_Month AS VARCHAR(10)) + '/' + CAST(@Calendar_Day AS VARCHAR(10)) + '/' + CAST(@Calendar_Year AS VARCHAR(10));
		SELECT @Day_of_Week = DATEPART(WEEKDAY, @Date_Counter);
		SELECT @Day_Name = CASE @Day_of_Week
								WHEN 1 THEN 'Sunday'
								WHEN 2 THEN 'Monday'
								WHEN 3 THEN 'Tuesday'
								WHEN 4 THEN 'Wednesday'
								WHEN 5 THEN 'Thursday'
								WHEN 6 THEN 'Friday'
								WHEN 7 THEN 'Saturday'
							END;
		SELECT @Day_of_Quarter = DATEDIFF(DAY, DATEADD(QUARTER, DATEDIFF(QUARTER, 0 , @Date_Counter), 0), @Date_Counter) + 1;
		SELECT @Day_of_Year = DATEPART(DAYOFYEAR, @Date_Counter);
		SELECT @Week_of_Month = DATEDIFF(WEEK, DATEADD(WEEK, DATEDIFF(WEEK, 0, DATEADD(MONTH, DATEDIFF(MONTH, 0, @Date_Counter), 0)), 0), @Date_Counter ) + 1;
		SELECT @Week_of_Quarter = DATEDIFF(DAY, DATEADD(QUARTER, DATEDIFF(QUARTER, 0, @Date_Counter), 0), @Date_Counter)/7 + 1;
		SELECT @Week_of_Year = DATEPART(WEEK, @Date_Counter);
		SELECT @Month_Name = CASE @Calendar_Month
								WHEN 1 THEN 'January'
								WHEN 2 THEN 'February'
								WHEN 3 THEN 'March'
								WHEN 4 THEN 'April'
								WHEN 5 THEN 'May'
								WHEN 6 THEN 'June'
								WHEN 7 THEN 'July'
								WHEN 8 THEN 'August'
								WHEN 9 THEN 'September'
								WHEN 10 THEN 'October'
								WHEN 11 THEN 'November'
								WHEN 12 THEN 'December'
							END;

		SELECT @First_Date_of_Week = DATEADD(DAY, -1 * @Day_of_Week + 1, @Date_Counter);
		SELECT @Last_Date_of_Week = DATEADD(DAY, 1 * (7 - @Day_of_Week), @Date_Counter);
		SELECT @First_Date_of_Month = DATEADD(DAY, -1 * DATEPART(DAY, @Date_Counter) + 1, @Date_Counter);
		SELECT @Last_Date_of_Month = EOMONTH(@Date_Counter);
		SELECT @First_Date_of_Quarter = DATEADD(QUARTER, DATEDIFF(QUARTER, 0, @Date_Counter), 0);
		SELECT @Last_Date_of_Quarter = DATEADD (DAY, -1, DATEADD(QUARTER, DATEDIFF(QUARTER, 0, @Date_Counter) + 1, 0));
		SELECT @First_Date_of_Year = DATEADD(YEAR, DATEDIFF(YEAR, 0, @Date_Counter), 0);
		SELECT @Last_Date_of_Year = DATEADD(DAY, -1, DATEADD(YEAR, DATEDIFF(YEAR, 0, @Date_Counter) + 1, 0));
		SELECT @Day_of_Week_in_Month = (@Calendar_Day + 6) / 7;
		SELECT @Day_of_Week_in_Year = (@Day_of_Year + 6) / 7;
		SELECT @Day_of_Week_in_Quarter = (@Day_of_Quarter + 6) / 7;
		SELECT @Is_Leap_Year = CASE
									WHEN @Calendar_Year % 4 <> 0 THEN 0
									WHEN @Calendar_Year % 100 <> 0 THEN 1
									WHEN @Calendar_Year % 400 <> 0 THEN 0
									ELSE 1
							   END;
		SELECT @First_Date_of_Next_Year = DATEADD(YEAR, 1, @First_Date_of_Year);
		SELECT @First_Date_of_Last_Year = DATEADD(YEAR, DATEDIFF(YEAR, 0, DATEADD(YEAR, -1, @Date_Counter)), 0);
		SELECT @First_Date_of_Two_Years_Ago = DATEADD(YEAR, DATEDIFF(YEAR, 0, DATEADD(YEAR, -2, @Date_Counter)), 0);
		
		SELECT @Merchant_First_Date_of_Year = DATEADD(DAY, 28 + CASE DATEPART(WEEKDAY, @First_Date_of_Year) WHEN 1 THEN 0 ELSE 8 - DATEPART(WEEKDAY, @First_Date_of_Year) END, @First_Date_of_Year);

		SELECT @Merchant_First_Date_of_Next_Year = DATEADD(DAY, 28 + CASE DATEPART(WEEKDAY, @First_Date_of_Next_Year) WHEN 1 THEN 0 ELSE 8 - DATEPART(WEEKDAY, @First_Date_of_Next_Year) END, @First_Date_of_Next_Year);
			
		SELECT @Merchant_First_Date_of_Last_Year = DATEADD(DAY, 28 + CASE DATEPART(WEEKDAY, @First_Date_of_Last_Year) WHEN 1 THEN 0 ELSE 8 - DATEPART(WEEKDAY, @First_Date_of_Last_Year) END, @First_Date_of_Last_Year);
		
		IF @Date_Counter < @Merchant_First_Date_of_Year
		BEGIN
			SELECT @Merchant_First_Date_of_Next_Year = @Merchant_First_Date_of_Year;
			SELECT @Merchant_First_Date_of_Year = @Merchant_First_Date_of_Last_Year;
			SELECT @Merchant_First_Date_of_Last_Year = DATEADD(DAY, 28 + CASE DATEPART(WEEKDAY, @First_Date_of_Two_Years_Ago) WHEN 1 THEN 0 ELSE 8 - DATEPART(WEEKDAY, @First_Date_of_Two_Years_Ago) END, @First_Date_of_Two_Years_Ago);
		END

		SELECT @Merchant_Last_Date_of_Year = DATEADD(DAY, -1, @Merchant_First_Date_of_Next_Year);

		SELECT @Merchant_Day_of_Year = CASE
										WHEN @Date_Counter >= @Merchant_First_Date_of_Year
											THEN DATEDIFF(DAY, @Merchant_First_Date_of_Year, @Date_Counter) + 1
										ELSE DATEDIFF(DAY, @Merchant_First_Date_of_Last_Year, @Date_Counter) + 1
									END;

		SELECT @Merchant_Week_of_Year = ((@Merchant_Day_of_Year - 1) / 7) + 1;

		SELECT @Days_in_Month = CASE
									WHEN @Calendar_Month IN (4, 6, 9, 11) THEN 30
									WHEN @Calendar_Month IN (1, 3, 5, 7, 8, 10, 12) THEN 31
									WHEN @Calendar_Month = 2 AND @Is_Leap_Year = 1 THEN 29
									ELSE 28
								END;
		SELECT @Merchant_Calendar_Month = CASE -- 4-5-4 monthly schedule
											WHEN @Merchant_Week_of_Year BETWEEN 1 AND 4 THEN 1 -- 4 weeks
											WHEN @Merchant_Week_of_Year BETWEEN 5 AND 9 THEN 2 -- 5 weeks
											WHEN @Merchant_Week_of_Year BETWEEN 10 AND 13 THEN 3 -- 4 weeks
											WHEN @Merchant_Week_of_Year BETWEEN 14 AND 17 THEN 4 -- 4 weeks
											WHEN @Merchant_Week_of_Year BETWEEN 18 AND 22 THEN 5 -- 5 weeks
											WHEN @Merchant_Week_of_Year BETWEEN 23 AND 26 THEN 6 -- 4 weeks
											WHEN @Merchant_Week_of_Year BETWEEN 27 AND 30 THEN 7 -- 4 weeks
											WHEN @Merchant_Week_of_Year BETWEEN 31 AND 35 THEN 8 -- 5 weeks
											WHEN @Merchant_Week_of_Year BETWEEN 36 AND 39 THEN 9 -- 4 weeks
											WHEN @Merchant_Week_of_Year BETWEEN 40 AND 43 THEN 10 -- 4 weeks
											WHEN @Merchant_Week_of_Year BETWEEN 44 AND 48 THEN 11 -- 5 weeks
											WHEN @Merchant_Week_of_Year BETWEEN 49 AND 53 THEN 12 -- 4 weeks (5 weeks during 4-5-4 leap years, in which case we add the final week, 53, into a 4-5-5 month)
									   END;
		SELECT @Merchant_Calendar_Quarter = CASE
											WHEN @Merchant_Calendar_Month BETWEEN 1 AND 3 THEN 1
											WHEN @Merchant_Calendar_Month BETWEEN 4 AND 6 THEN 2
											WHEN @Merchant_Calendar_Month BETWEEN 7 AND 9 THEN 3
											WHEN @Merchant_Calendar_Month BETWEEN 10 AND 12 THEN 4
										 END;
		SELECT @Is_Merchant_Leap_Year = CASE DATEDIFF(DAY, @Merchant_First_Date_of_Year, @Merchant_Last_Date_of_Year)
										WHEN 363 THEN 0
										ELSE 1
									  END;
		SELECT @Merchant_First_Date_of_Month = CASE
												WHEN @Merchant_Day_of_Year <= 28 THEN DATEADD(DAY, -1 * @Merchant_Day_of_Year + 1, @Date_Counter) -- Month 1 (Start)
												WHEN @Merchant_Day_of_Year BETWEEN 29 AND 63 THEN DATEADD(DAY, -1 * (@Merchant_Day_of_Year - 28) + 1, @Date_Counter) -- Month 2 (4 weeks)
												WHEN @Merchant_Day_of_Year BETWEEN 64 AND 91 THEN DATEADD(DAY, -1 * (@Merchant_Day_of_Year - 63) + 1, @Date_Counter) -- Month 3 (5 weeks)
												WHEN @Merchant_Day_of_Year BETWEEN 92 AND 119 THEN DATEADD(DAY, -1 * (@Merchant_Day_of_Year - 91) + 1, @Date_Counter) -- Month 4 (4 weeks)
												WHEN @Merchant_Day_of_Year BETWEEN 120 AND 154 THEN DATEADD(DAY, -1 * (@Merchant_Day_of_Year - 119) + 1, @Date_Counter) -- Month 5 (4 weeks)
												WHEN @Merchant_Day_of_Year BETWEEN 155 AND 182 THEN DATEADD(DAY, -1 * (@Merchant_Day_of_Year - 154) + 1, @Date_Counter) -- Month 6 (5 weeks)
												WHEN @Merchant_Day_of_Year BETWEEN 183 AND 210 THEN DATEADD(DAY, -1 * (@Merchant_Day_of_Year - 182) + 1, @Date_Counter) -- Month 7 (4 weeks)
												WHEN @Merchant_Day_of_Year BETWEEN 211 AND 245 THEN DATEADD(DAY, -1 * (@Merchant_Day_of_Year - 210) + 1, @Date_Counter) -- Month 8 (4 weeks)
												WHEN @Merchant_Day_of_Year BETWEEN 246 AND 273 THEN DATEADD(DAY, -1 * (@Merchant_Day_of_Year - 245) + 1, @Date_Counter) -- Month 9 (5 weeks)
												WHEN @Merchant_Day_of_Year BETWEEN 274 AND 301 THEN DATEADD(DAY, -1 * (@Merchant_Day_of_Year - 273) + 1, @Date_Counter) -- Month 10 (4 weeks)
												WHEN @Merchant_Day_of_Year BETWEEN 302 AND 336 THEN DATEADD(DAY, -1 * (@Merchant_Day_of_Year - 301) + 1, @Date_Counter) -- Month 11 (4 weeks)
												WHEN @Merchant_Day_of_Year >= 336 THEN DATEADD(DAY, -1 * (@Merchant_Day_of_Year - (CASE WHEN @Is_Merchant_Leap_Year = 0 THEN 336 ELSE 343 END)) + 1, @Date_Counter) -- Month 12 (5 weeks) through End of year (4 or 5 weeks, depending on 4-5-4 leap year status)
											 END;
		SELECT @Merchant_Last_Date_of_Month = CASE
												WHEN @Merchant_Week_of_Year BETWEEN 1 AND 4 THEN DATEADD(DAY, 27, @Merchant_First_Date_of_Month) -- 4 weeks
												WHEN @Merchant_Week_of_Year BETWEEN 5 AND 9 THEN DATEADD(DAY, 34, @Merchant_First_Date_of_Month) -- 5 weeks
												WHEN @Merchant_Week_of_Year BETWEEN 10 AND 13 THEN DATEADD(DAY, 27, @Merchant_First_Date_of_Month) -- 4 weeks
												WHEN @Merchant_Week_of_Year BETWEEN 14 AND 17 THEN DATEADD(DAY, 27, @Merchant_First_Date_of_Month) -- 4 weeks
												WHEN @Merchant_Week_of_Year BETWEEN 18 AND 22 THEN DATEADD(DAY, 34, @Merchant_First_Date_of_Month) -- 5 weeks
												WHEN @Merchant_Week_of_Year BETWEEN 23 AND 26 THEN DATEADD(DAY, 27, @Merchant_First_Date_of_Month) -- 4 weeks
												WHEN @Merchant_Week_of_Year BETWEEN 27 AND 30 THEN DATEADD(DAY, 27, @Merchant_First_Date_of_Month) -- 4 weeks
												WHEN @Merchant_Week_of_Year BETWEEN 31 AND 35 THEN DATEADD(DAY, 34, @Merchant_First_Date_of_Month) -- 5 weeks
												WHEN @Merchant_Week_of_Year BETWEEN 36 AND 39 THEN DATEADD(DAY, 27, @Merchant_First_Date_of_Month) -- 4 weeks
												WHEN @Merchant_Week_of_Year BETWEEN 40 AND 43 THEN DATEADD(DAY, 27, @Merchant_First_Date_of_Month) -- 4 weeks
												WHEN @Merchant_Week_of_Year BETWEEN 44 AND 48 THEN DATEADD(DAY, 34, @Merchant_First_Date_of_Month) -- 5 weeks
												WHEN @Merchant_Week_of_Year BETWEEN 49 AND 53 THEN CASE -- 4 weeks (5 weeks during 4-5-4 leap years, in which case we add the final week, 53, into a 4-5-5 month)
																									WHEN @Is_Merchant_Leap_Year = 0 THEN DATEADD(DAY, 27, @Merchant_First_Date_of_Month)
																									WHEN @Is_Merchant_Leap_Year = 1 THEN DATEADD(DAY, 34, @Merchant_First_Date_of_Month)
																								 END 
											END;
		SELECT @Merchant_First_Date_of_Quarter = CASE
												WHEN @Merchant_Day_of_Year <= 91 THEN DATEADD(DAY, -1 * @Merchant_Day_of_Year + 1, @Date_Counter) -- Month 1 (Start)
												WHEN @Merchant_Day_of_Year BETWEEN 92 AND 182 THEN DATEADD(DAY, -1 * @Merchant_Day_of_Year + 92, @Date_Counter) -- Month 4 (4 weeks)
												WHEN @Merchant_Day_of_Year BETWEEN 183 AND 273 THEN DATEADD(DAY, -1 * @Merchant_Day_of_Year  + 183, @Date_Counter) -- Month 7 (4 weeks)
												WHEN @Merchant_Day_of_Year >= 274 THEN DATEADD(DAY, -1 * @Merchant_Day_of_Year + 274, @Date_Counter) -- Month 10 (4 weeks)
											   END;
		SELECT @Merchant_Last_Date_of_Quarter = CASE
												WHEN @Merchant_Calendar_Quarter = 1 THEN DATEADD(DAY, 90, @Merchant_First_Date_of_Quarter)
												WHEN @Merchant_Calendar_Quarter = 2 THEN DATEADD(DAY, 90, @Merchant_First_Date_of_Quarter)
												WHEN @Merchant_Calendar_Quarter = 3 THEN DATEADD(DAY, 90, @Merchant_First_Date_of_Quarter)
												WHEN @Merchant_Calendar_Quarter = 4 THEN CASE
																				WHEN @Is_Merchant_Leap_Year = 0 THEN DATEADD(DAY, 90, @Merchant_First_Date_of_Quarter)
																				ELSE DATEADD(DAY, 97, @Merchant_First_Date_of_Quarter)
																			  END
											  END;

		SELECT @Merchant_First_Date_of_Week = DATEADD(DAY, -1 * ((DATEDIFF(DAY, @Merchant_First_Date_of_Month, @Date_Counter) % 7) + 1) + 1, @Date_Counter);
		SELECT @Merchant_Last_Date_of_Week = DATEADD(DAY, 6, @Merchant_First_Date_of_Week);

		SELECT @ISO_Week_of_Year = DATEPART(ISO_WEEK, @Date_Counter);

		SELECT @ISO_First_Date_of_Week = CASE
											WHEN DATEPART(WEEKDAY, @Date_Counter) = 2 THEN @Date_Counter
											WHEN DATEPART(WEEKDAY, @Date_Counter) = 1 THEN DATEADD(DAY, -6, @Date_Counter)
											ELSE DATEADD(DAY, -1 * (DATEPART(WEEKDAY, @Date_Counter) - 2), @Date_Counter)
										 END;
		SELECT @ISO_Last_Date_of_Week = DATEADD(DAY, 6, @ISO_First_Date_of_Week);

		SELECT @ISO_Day_of_Week =	CASE
										WHEN DATEPART(WEEKDAY, @Date_Counter) = 1 THEN 7
										ELSE DATEPART(WEEKDAY, @Date_Counter) -1
									END

		SELECT @ISO_Day_of_Year = (@ISO_Week_of_Year - 1) * 7 + @ISO_Day_of_Week;
		
		SELECT @ISO_Year = CASE
								WHEN @Week_of_Year > 49 AND @ISO_Week_of_Year < 3 THEN @Calendar_Year + 1
								WHEN @Week_of_Year < 3 AND @ISO_Week_of_Year > 49 THEN @Calendar_Year - 1
								ELSE @Calendar_Year
							 END;

		SELECT @ISO_String = CAST(@ISO_Year AS VARCHAR(6)) + '-W' + CAST(@ISO_Week_of_Year AS VARCHAR(2)) + '-' + CAST(@ISO_Day_of_Week AS VARCHAR(1));

		-- These fiscal year columns assume a calendar that begins on October 1st and ends on September 31st.  It is otherwise unchanged.
		SELECT @Fiscal_First_Date_of_Year = CASE
												WHEN @Calendar_Month IN (10, 11, 12) THEN DATEADD(DAY, -1 * @Day_of_Quarter + 1, @Date_Counter)
												ELSE DATEADD(DAY, -1 * @Day_of_Year - 91, @Date_Counter)
											END
		SELECT @Fiscal_Last_Date_of_Year = DATEADD(DAY, -1, DATEADD(YEAR, 1, @Fiscal_First_Date_of_Year));
		SELECT @Fiscal_Day_of_Year = DATEDIFF(DAY, @Fiscal_First_Date_of_Year, @Date_Counter) + 1;

		SELECT @Fiscal_Week_of_Year = (@Fiscal_Day_of_Year + DATEPART(WEEKDAY, @Fiscal_First_Date_of_Year) - DATEPART(WEEKDAY, @Date_Counter)) / 7 + 1;

		SELECT @Fiscal_Quarter = CASE
									WHEN @Calendar_Quarter = 4 THEN 1
									ELSE @Calendar_Quarter + 1
								 END;
		SELECT @Fiscal_Month = CASE
									WHEN @Calendar_Month BETWEEN 10 AND 12 THEN @Calendar_Month - 9
									ELSE @Calendar_Month + 3
							   END;

		INSERT INTO dbo.Dim_Date
			(Calendar_Date, Calendar_Date_String, Calendar_Month, Calendar_Day, Calendar_Year, Calendar_Quarter, Day_Name, Day_of_Week, Day_of_Week_in_Month,
				Day_of_Week_in_Year, Day_of_Week_in_Quarter, Day_of_Quarter, Day_of_Year, Week_of_Month, Week_of_Quarter, Week_of_Year, Month_Name,
				First_Date_of_Week, Last_Date_of_Week, First_Date_of_Month, Last_Date_of_Month, First_Date_of_Quarter, Last_Date_of_Quarter, First_Date_of_Year,
				Last_Date_of_Year, Is_Leap_Year, Days_in_Month, Merchant_Day_of_Year, Merchant_Week_of_Year, Merchant_Calendar_Month, Merchant_Calendar_Quarter,
				Merchant_First_Date_of_Week, Merchant_Last_Date_of_Week, Merchant_First_Date_of_Month, Merchant_Last_Date_of_Month,
				Merchant_First_Date_of_Quarter, Merchant_Last_Date_of_Quarter, Merchant_First_Date_of_Year, Merchant_Last_Date_of_Year, ISO_Day_of_Week, ISO_Day_of_Year,
				ISO_Week_of_Year, ISO_First_Date_of_Week, ISO_Last_Date_of_Week, ISO_String, Fiscal_Day_of_Year, Fiscal_Week_of_Year, Fiscal_Quarter,
				Fiscal_Month, Fiscal_First_Date_of_Year, Fiscal_Last_Date_of_Year)
		SELECT
			@Date_Counter AS Calendar_Date,
			@Calendar_Date_String AS Calendar_Date_String,
			@Calendar_Month AS Calendar_Month,
			@Calendar_Day AS Calendar_Day,
			@Calendar_Year AS Calendar_Year,
			@Calendar_Quarter AS Calendar_Quarter,
			@Day_Name AS Day_Name,
			@Day_of_Week AS Day_of_Week,
			@Day_of_Week_in_Month AS Day_of_Week_in_Month,
			@Day_of_Week_in_Year AS Day_of_Week_in_Year,
			@Day_of_Week_in_Quarter AS Day_of_Week_in_Quarter,
			@Day_of_Quarter AS Day_of_Quarter,
			@Day_of_Year AS Day_of_Year,
			@Week_of_Month AS Week_of_Month,
			@Week_of_Quarter AS Week_of_Quarter,
			@Week_of_Year AS Week_of_Year,
			@Month_Name AS Month_Name,
			@First_Date_of_Week AS First_Date_of_Week,
			@Last_Date_of_Week AS Last_Date_of_Week,
			@First_Date_of_Month AS First_Date_of_Month,
			@Last_Date_of_Month AS Last_Date_of_Month,
			@First_Date_of_Quarter AS First_Date_of_Quarter,
			@Last_Date_of_Quarter AS Last_Date_of_Quarter,
			@First_Date_of_Year AS First_Date_of_Year,
			@Last_Date_of_Year AS Last_Date_of_Year,
			@Is_Leap_Year AS Is_Leap_Year,
			@Days_in_Month AS Days_in_Month,
			@Merchant_Day_of_Year AS Merchant_Day_of_Year,
			@Merchant_Week_of_Year AS Merchant_Week_of_Year,
			@Merchant_Calendar_Month AS Merchant_Calendar_Month,
			@Merchant_Calendar_Quarter AS Merchant_Calendar_Quarter,
			@Merchant_First_Date_of_Week AS Merchant_First_Date_of_Week,
			@Merchant_Last_Date_of_Week AS Merchant_Last_Date_of_Week,
			@Merchant_First_Date_of_Month AS Merchant_First_Date_of_Month,
			@Merchant_Last_Date_of_Month AS Merchant_Last_Date_of_Month,
			@Merchant_First_Date_of_Quarter AS Merchant_First_Date_of_Quarter,
			@Merchant_Last_Date_of_Quarter AS Merchant_Last_Date_of_Quarter,
			@Merchant_First_Date_of_Year AS Merchant_First_Date_of_Year,
			@Merchant_Last_Date_of_Year AS Merchant_Last_Date_of_Year,
			@ISO_Day_of_Week AS ISO_Day_of_Week,
			@ISO_Day_of_Year AS ISO_Day_of_Year,
			@ISO_Week_of_Year AS ISO_Week_of_Year,
			@ISO_First_Date_of_Week AS ISO_First_Date_of_Week,
			@ISO_Last_Date_of_Week AS ISO_Last_Date_of_Week,
			@ISO_String AS ISO_String,
			@Fiscal_Day_of_Year AS Fiscal_Day_of_Year,
			@Fiscal_Week_of_Year AS Fiscal_Week_of_Year,
			@Fiscal_Quarter AS Fiscal_Quarter,
			@Fiscal_Month AS Fiscal_Month,
			@Fiscal_First_Date_of_Year AS Fiscal_First_Date_of_Year,
			@Fiscal_Last_Date_of_Year AS Fiscal_Last_Date_of_Year;
	
		SELECT @Date_Counter = DATEADD(DAY, 1, @Date_Counter);
	END
END
GO

EXEC dbo.Populate_Dim_Date '1/1/2015', '12/31/2020'

SELECT
	*
FROM dbo.Dim_Date;
